package com.hai.service.impl;

import com.hai.dao.HighDiscountPackageMapper;
import com.hai.dao.HighDiscountPackageRecordMapper;
import com.hai.entity.*;
import com.hai.service.HighDiscountPackageService;
import com.hai.service.HighDiscountPackageService;
import org.apache.commons.collections4.MapUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @serviceName HighDiscountPackageServiceImpl.java
 * @author Sum1Dream
 * @version 1.0.0
 * @Description // 优惠券包 接口
 * @createTime 12:21 下午 2021/11/25
 **/
@Service("highDiscountPackageService")
public class HighDiscountPackageServiceImpl implements HighDiscountPackageService {

    @Resource
    private HighDiscountPackageMapper highDiscountPackageMapper;

    @Resource
    private HighDiscountPackageRecordMapper highDiscountPackageRecordMapper;

    @Override
    public List<HighDiscountPackage> getDiscountPackageList(Map<String, Object> map) {

        HighDiscountPackageExample example = new HighDiscountPackageExample();
        HighDiscountPackageExample.Criteria criteria = example.createCriteria();

        if (MapUtils.getInteger(map, "companyId") != null) {
            criteria.andCompanyIdEqualTo(MapUtils.getInteger(map, "companyId"));
        }

        if (MapUtils.getInteger(map, "usingAttribution") != null) {
            criteria.andUsingAttributionEqualTo(MapUtils.getInteger(map, "usingAttribution"));
        }

        if (MapUtils.getString(map, "title") != null) {
            criteria.andTitleLike("%" + MapUtils.getString(map, "title") + "%");
        }

        if (MapUtils.getInteger(map, "salesType") != null) {
            criteria.andSalesTypeEqualTo(MapUtils.getInteger(map, "salesType"));
        }

        if (MapUtils.getInteger(map, "status") != null) {
            criteria.andStatusEqualTo(MapUtils.getInteger(map, "status"));
        } else {
            criteria.andStatusNotEqualTo(4);
        }

        example.setOrderByClause("created_time desc");

        return highDiscountPackageMapper.selectByExample(example);
    }

    @Override
    public HighDiscountPackage findDiscountPackageById(Integer id) {
        return highDiscountPackageMapper.selectByPrimaryKey(id);
    }

    @Override
    public HighDiscountPackage getCallExclusive(Integer usingAttribution , Integer companyId) {
        HighDiscountPackageExample example = new HighDiscountPackageExample();
        HighDiscountPackageExample.Criteria criteria = example.createCriteria();
        criteria.andUsingAttributionEqualTo(usingAttribution).andStatusNotEqualTo(4).andCompanyIdEqualTo(companyId);;
        return highDiscountPackageMapper.selectByExample(example).get(0);
    }

    @Override
    public void insertDiscountPackage(HighDiscountPackage highDiscountPackage) {
        highDiscountPackageMapper.insert(highDiscountPackage);
    }

    @Override
    public void updateDiscountPackage(HighDiscountPackage highDiscountPackage) {
        highDiscountPackageMapper.updateByPrimaryKey(highDiscountPackage);
    }

    @Override
    public List<HighDiscountPackageRecord> getDiscountPackageRecordList(Map<String, Object> map) {

        HighDiscountPackageRecordExample example = new HighDiscountPackageRecordExample();
        HighDiscountPackageRecordExample.Criteria criteria = example.createCriteria();

        if (MapUtils.getInteger(map, "companyId") != null) {
            criteria.andCompanyIdEqualTo(MapUtils.getInteger(map, "companyId"));
        }

        if (MapUtils.getInteger(map, "usingAttribution") != null) {
            criteria.andUsingAttributionEqualTo(MapUtils.getInteger(map, "usingAttribution"));
        }

        if (MapUtils.getString(map, "discountPackageTitle") != null) {
            criteria.andDiscountPackageTitleLike("%" + MapUtils.getString(map, "discountPackageTitle") + "%");
        }

        if (MapUtils.getString(map, "recordNo") != null) {
            criteria.andRecordNoLike("%" + MapUtils.getString(map, "recordNo") + "%");
        }

        if (MapUtils.getInteger(map, "salesType") != null) {
            criteria.andSalesTypeEqualTo(MapUtils.getInteger(map, "salesType"));
        }

        if (MapUtils.getString(map, "phone") != null) {
            criteria.andExt1Like("%" + MapUtils.getString(map, "phone") + "%");
        }

        return highDiscountPackageRecordMapper.selectByExample(example);
    }
}
