package com.hai.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.utils.DateUtil;
import com.hai.common.utils.IDGenerator;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.config.HuiLianTongUnionCardConfig;
import com.hai.dao.HighDiscountPackageRecordMapper;
import com.hai.dao.HighDiscountUserRelMapper;
import com.hai.entity.*;
import com.hai.service.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

@Service("hltUnionCardVipService")
public class HltUnionCardVipServiceImpl implements HltUnionCardVipService {

    @Resource
    private HighDiscountPackageRecordMapper highDiscountPackageRecordMapper;

    @Resource
    private HighUserService highUserService;

    @Resource
    private HighDiscountPackageService highDiscountPackageService;

    @Resource
    private CommonService commonService;

    @Resource
    private BsCompanyService bsCompanyService;

    @Resource
    private HighDiscountPackageDetailsService highDiscountPackageDetailsService;

    @Resource
    private HighDiscountAgentRelService highDiscountAgentRelService;

    @Resource
    private HighDiscountAgentCodeService highDiscountAgentCodeService;

    @Resource
    private HighDiscountUserRelMapper highDiscountUserRelMapper;

    @Resource
    private HighDiscountService highDiscountService;


    @Override
    @Transactional(propagation= Propagation.REQUIRES_NEW,isolation= Isolation.SERIALIZABLE)
    public JSONObject GetMembershipLevel(String phone , String regionId)   throws Exception  {

        SecRegion region = commonService.getParentByRegion(Long.parseLong(regionId));

        BsCompany bsCompany = bsCompanyService.selectCompanyByRegion(region.getRegionId().toString());

        JSONObject consumptionRecord = HuiLianTongUnionCardConfig.TCheckEMsgUnionCardVipRights(phone);
        JSONObject cardInfoObject = HuiLianTongUnionCardConfig.resolveResponse(consumptionRecord.getString("data"));


        HighUser highUser = highUserService.findByPhone(phone);

        if (highUser == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "当前用户不存在");
        }

        boolean isVip = false;

        // 判断用户是否有会员权益
        for (Object dataObject : cardInfoObject.getJSONArray("data")) {
            JSONObject d = (JSONObject) dataObject;
            if (d.getInteger("level").equals(12)) {
                isVip = true;
            }
        }

        // 判断汇联通会员等级 1-E信通会员，2-工会卡会员，12-E信通和工会卡会员
        if (isVip) {

            List<HighDiscountPackageRecord> list = highDiscountPackageRecordMapper.CheckCouponsMonth(highUser.getId() , 1);

            // 判断当月是否赠送会员权益
            if (list.size() > 0 ) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "用户当月已赠送");
            } else {

                Map<String,Object> map = new HashMap<>();
                map.put("companyId", bsCompany.getId());
                map.put("usingAttribution", 1);
                map.put("salesType", 2);

                // 查询话费专属优惠券包
                List<HighDiscountPackage> highDiscountPackages =  highDiscountPackageService.getDiscountPackageList(map);

                if (highDiscountPackages.size() == 0) {
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "暂未配置专属优惠券包");
                }

                HighDiscountPackage highDiscountPackage = highDiscountPackages.get(0);

                Map<String , Object> mapRule = new HashMap<>();
                mapRule.put("discountPackageId", highDiscountPackage.getId());

                // 查询优惠券包规则
                List<HighDiscountPackageDetails> discountPackageDetailsList = highDiscountPackageDetailsService.getDiscountPackageDetailsList(mapRule);

                // 循环 优惠券包规则详情 列表
                for (HighDiscountPackageDetails detailsList: discountPackageDetailsList) {

                    // 查询代理商与优惠券关系 列表
                    List<HighDiscountAgentRel> discountAgentRels = highDiscountAgentRelService.getRelByDiscountId(detailsList.getDiscountId().longValue());

                    HighDiscountAgentRel discountAgentRel = new HighDiscountAgentRel();

                    for (HighDiscountAgentRel discountAgentRelList: discountAgentRels) {
                        // 判断当前是否足够库存赠送
                        if (discountAgentRelList.getStockCount() > detailsList.getNum()) {
                            discountAgentRel = discountAgentRelList;
                            break;
                        }
                    }

                    List<HighDiscountAgentCode> discountAgentCodeList = highDiscountAgentCodeService.getDiscountCodeByDiscountAgentId(discountAgentRel.getId());

                    // 赠送优惠券
                    for (HighDiscountAgentCode discountAgentCode : discountAgentCodeList.subList(0, detailsList.getNum())) {
                        HighDiscountUserRel highDiscountUserRel = new HighDiscountUserRel();
                        highDiscountUserRel.setDiscountId(discountAgentRel.getDiscountId());
                        highDiscountUserRel.setUserId(highUser.getId());
                        highDiscountUserRel.setAgentId(discountAgentRel.getAgentId());
                        highDiscountUserRel.setDiscountAgentCodeId(discountAgentCode.getId());
                        highDiscountUserRel.setStatus(1);
                        highDiscountUserRel.setCreateTime(new Date());

                        HighDiscount highDiscount  = highDiscountService.getDiscountById(discountAgentRel.getDiscountId());

                        // 计算使用有效期
                        Calendar userEndTime = Calendar.getInstance();
                        userEndTime.setTime(new Date());
                        userEndTime.set(Calendar.HOUR_OF_DAY, 23);
                        userEndTime.set(Calendar.MINUTE, 59);
                        userEndTime.set(Calendar.SECOND, 59);
                        userEndTime.add(Calendar.DATE, highDiscount.getEffectiveDay());
                        highDiscountUserRel.setUseEndTime(userEndTime.getTime());

                        // 修改优惠券二维码状态
                        discountAgentCode.setStatus(2);
                        highDiscountAgentCodeService.updateCode(discountAgentCode);

                        // 新增优惠券与用户的绑定关系
                        highDiscountUserRelMapper.insert(highDiscountUserRel);
                    }

                }

                // 增加赠送记录
                HighDiscountPackageRecord discountPackageRecord = new HighDiscountPackageRecord();
                discountPackageRecord.setDiscountPackageId(highDiscountPackage.getId());
                discountPackageRecord.setDiscountPackageTitle(highDiscountPackage.getTitle());
                discountPackageRecord.setCompanyId(highDiscountPackage.getCompanyId());
                discountPackageRecord.setStatus(1);
                discountPackageRecord.setUsingAttribution(1);
                discountPackageRecord.setRecordNo("DP" + DateUtil.date2String(new Date(),"yyyyMMddHHmmss") + IDGenerator.nextId(5) );
                discountPackageRecord.setSalesType(2);
                discountPackageRecord.setUserId(highUser.getId().intValue());
                discountPackageRecord.setExt1(highUser.getPhone());
                discountPackageRecord.setCreatedTime(new Date());
                highDiscountPackageRecordMapper.insert(discountPackageRecord);
                }

            }


        return null;
    }
}
